"""
This was lifted straight from the scons documentation

http://www.scons.org/doc/HTML/scons-user.html#AEN2061

on September 5th, 2013

I've made a few modifications as needed.

Legal crap:
Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
"""

import atexit


def bf_to_str(bf):
    """Convert an element of GetBuildFailures() to a string
    in a useful way."""
    import SCons.Errors

    if bf is None:  # unknown targets product None in list
        return "(unknown tgt)"
    elif isinstance(bf, SCons.Errors.StopError):
        return str(bf)
    elif bf.node:
        return str(bf.node) + ": " + bf.errstr
    elif bf.filename:
        return bf.filename + ": " + bf.errstr
    return "unknown failure: " + bf.errstr


def build_status():
    """Convert the build status to a 2-tuple, (status, msg)."""
    from SCons.Script import GetBuildFailures

    bf = GetBuildFailures()
    if bf:
        # bf is normally a list of build failures; if an element is None,
        # it's because of a target that scons doesn't know anything about.
        status = "failed"
        failures_message = "\n".join(
            ["Failed building %s" % bf_to_str(x) for x in bf if x is not None]
        )
    else:
        # if bf is None, the build completed successfully.
        status = "ok"
        failures_message = ""
    return (status, failures_message)


def display_build_status():
    """Display the build status.  Called by atexit.
    Here you could do all kinds of complicated things."""
    if GetOption("clean") or GetOption("help"):
        return

    status, failures_message = build_status()
    if status == "failed":
        print
        print "FAILED!!!!"  # could display alert, ring bell, etc.
    elif status == "ok":
        print
        print "Build succeeded."
    print failures_message


atexit.register(display_build_status)
